module Innologix
  class Client
    attr_accessor :client_id,
                  :client_secret,
                  :innologix_url,
                  :access_token,
                  :auth,
                  :logger,
                  :sso_url,
                  :timeout,
                  :default_headers

    def initialize (options = {})
      @client_id = options[:client_id] || Innologix.config['client_id']
      @client_secret = options[:client_secret] || Innologix.config['client_secret']
      @innologix_url = options[:innologix_url] || Innologix.config['innologix_url']
      @sso_url = options[:sso_url] || Innologix.config['sso_url']
      @timeout = options[:timeout] || Innologix.config['timeout']
      @default_headers = {:'Content-Type' => "application/json",
                          :'Authorization' => 'Bearer ' + current_token}
      return true
    end

    def self.set_user_id(user_id)
      @default.default_headers = {:'Content-Type' => "application/json",
                                  :'X-User-ID' => user_id,
                                  :'Authorization' => 'Bearer ' + @default.access_token}
    end

    def self.default
      @default ||= Client.new
    end

    def current_token
      @access_token ||= get_token
    end

    def get_token
      url = @sso_url + '/oauth/access_token'
      begin
        RestClient.post url, {client_id: client_id, client_secret: client_secret, grant_type: 'client_credentials'} do |response, request, result, &block|
          case response.code
            when 200
              result = JSON.parse response, :symbolize_names => true
              if result[:access_token].nil?
                nil
              else
                result[:access_token]
              end
            else
              nil
          end
        end
      rescue Exception => e
        Innologix::Logger.error e.message
        nil
      end
    end

    def call_api(path, method, options = {})
      url = innologix_url + path
      header_params ||= @default_headers.merge(options[:header_params] || {})
      query_params = options[:query_params] || {}
      header_params[:params] = query_params
      form_params = options[:form_params] || {}
      RestClient::Request.execute(method: method.to_sym.downcase,
                                  url: url,
                                  payload: form_params,
                                  timeout: timeout,
                                  :verify_ssl => false,
                                  headers: header_params) do |response, request, result, &block|

        JSON.parse response, :symbolize_names => true
      end
    rescue Exception => e
      {error: 500, message: e.message}
    end
  end
end
