module Innologix
  class DeviceType
    attr_accessor :id
    attr_accessor :name
    attr_accessor :supervisor_id
    attr_accessor :protocol
    attr_accessor :tags
    attr_accessor :created_at
    attr_accessor :updated_at

    attr_accessor :client
    attr_accessor :error

    def initialize(h = {})
      h.each { |k, v| public_send("#{k}=", v) }
      @client = Client.default
    end

    def list(offset = 0, limit = 10)
      path = '/device_types'
      method = 'get'
      options = {query_params: {offset: offset, limit: limit}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        list =[]
        result[:device_types].each do |device_type|
          list.push(from_hash(device_type))
        end
        meta = OpenStruct.new
        meta.offset = result[:meta][:offset]
        meta.limit = result[:meta][:limit]
        meta.total = result[:meta][:total]

        result = OpenStruct.new
        result.device_types = list
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def get(id)
      path = '/device_types/' + id.to_s
      method = 'get'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def create
      path = '/device_types'
      method = 'post'
      options = {form_params: {device_type: {name: name, supervisor_id: supervisor_id, protocol: protocol, tags: tags}}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update
      path = '/device_types/' + id.to_s
      method = 'put'
      options = {form_params: {device_type: {name: name, supervisor_id: supervisor_id, protocol: protocol, tags: tags}}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def delete
      path = '/device_types/' + id.to_s
      method = 'delete'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def from_hash(attributes)
      device_type = Innologix::DeviceType.new
      device_type.id = attributes[:id]
      device_type.name = attributes[:name]
      device_type.supervisor_id = attributes[:supervisor_id]
      device_type.protocol = attributes[:protocol]
      device_type.tags = attributes[:tags]
      device_type.created_at = attributes[:created_at]
      device_type.updated_at = attributes[:updated_at]
      device_type
    end
  end
end