module Innologix
  class Group
    attr_accessor :id
    attr_accessor :name
    attr_accessor :supervisor_id
    attr_accessor :created_at
    attr_accessor :updated_at

    attr_accessor :supervisor
    attr_accessor :m2ms

    attr_accessor :client
    attr_accessor :error

    def initialize(h = {})
      h.each { |k, v| public_send("#{k}=", v) }
      @client = Client.default
    end

    def list(offset = 0, limit = 10)
      path = '/groups'
      method = 'get'
      options = {query_params: {offset: offset, limit: limit}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        list =[]
        result[:groups].each do |group|
          list.push(from_hash(group))
        end
        meta = OpenStruct.new
        meta.offset = result[:meta][:offset]
        meta.limit = result[:meta][:limit]
        meta.total = result[:meta][:total]

        result = OpenStruct.new
        result.groups = list
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def get(id)
      path = '/groups/' + id.to_s
      method = 'get'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def create
      path = '/groups'
      method = 'post'
      form_params = {name: name, supervisor_id: supervisor_id}
      options = {form_params: {group: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update
      path = '/groups/' + id.to_s
      method = 'put'
      form_params = {name: name, supervisor_id: supervisor_id}
      options = {form_params: {group: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def delete
      path = '/groups/' + id.to_s
      method = 'delete'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def from_hash(attributes)
      group = Innologix::Group.new
      group.id = attributes[:id]
      group.name = attributes[:name]
      group.supervisor_id = attributes[:supervisor_id]
      group.created_at = attributes[:created_at]
      group.updated_at = attributes[:updated_at]

      group.m2ms = []
      if attributes[:m2ms] != nil
        attributes[:m2ms].each do |m2m|
          group.m2ms.push(Innologix::M2m.new(m2m))
        end
      end
      group
    end
  end
end
