module Innologix
  class M2m
    attr_accessor :id
    attr_accessor :name
    attr_accessor :ip
    attr_accessor :status
    attr_accessor :group_id
    attr_accessor :created_at
    attr_accessor :updated_at

    attr_accessor :group

    attr_accessor :client
    attr_accessor :error

    def initialize(h = {})
      h.each { |k, v| public_send("#{k}=", v) }
      @client = Client.default
    end

    def list(offset = 0, limit = 10, params = {})
      path = '/m2ms'
      method = 'get'
      options = {query_params: {offset: offset, limit: limit}.merge(params)}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        list =[]
        result[:m2ms].each do |m2m|
          list.push(from_hash(m2m))
        end
        meta = OpenStruct.new
        meta.offset = result[:meta][:offset]
        meta.limit = result[:meta][:limit]
        meta.total = result[:meta][:total]

        result = OpenStruct.new
        result.m2ms = list
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def get(id)
      path = '/m2ms/' + id.to_s
      method = 'get'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def create
      path = '/m2ms'
      method = 'post'
      form_params = {name: name, ip: ip, group_id: group_id}
      options = {form_params: {m2m: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update
      path = '/m2ms/' + id.to_s
      method = 'put'
      form_params = {name: name, ip: ip, group_id: group_id}
      options = {form_params: {m2m: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def delete
      path = '/m2ms/' + id.to_s
      method = 'delete'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def add_devices(devices = [], callback = nil)
      path = '/m2ms/' + id.to_s + '/add_devices'
      method = 'patch'
      form_params = {devices: devices, callback: callback}
      options = {form_params: form_params}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def remove_devices(devices = [], callback = nil)
      path = '/m2ms/' + id.to_s + '/remove_devices'
      method = 'patch'
      form_params = {devices: devices, callback: callback}
      options = {form_params: form_params}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def restart(callback = nil)
      path = '/m2ms/' + id.to_s + '/restart'
      method = 'patch'
      form_params = {callback: callback}
      options = {form_params: form_params}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        result[:message]
      else
        RequestError.new(result)
      end
    end

    def check_status(callback = nil)
      path = '/m2ms/' + id.to_s + '/check_status'
      method = 'patch'
      form_params = {callback: callback}
      options = {form_params: form_params}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        result[:message]
      else
        RequestError.new(result)
      end
    end

    def from_hash(attributes)
      m2m = Innologix::M2m.new
      m2m.id = attributes[:id]
      m2m.name = attributes[:name]
      m2m.ip = attributes[:ip]
      m2m.status = attributes[:status]
      m2m.group_id = attributes[:group_id]
      m2m.created_at = attributes[:created_at]
      m2m.updated_at = attributes[:updated_at]
      if attributes[:group] != nil
        m2m.group = Innologix::Group.new(attributes[:group])
      end
      m2m
    end

    def devices(offset = 0, limit = 10, params = {})
      path = '/devices/list'
      method = 'get'
      options = {query_params: {m2m_id: self.id, offset: offset, limit: limit}.merge(params)}
      result = client.call_api(path, method, options)
      list =[]
      if result[:error].nil?
        result[:devices].each do |device|
          list.push(Innologix::Device.new(device))
        end
      end
      meta = OpenStruct.new
      meta.offset = result[:meta][:offset]
      meta.limit = result[:meta][:limit]
      meta.total = result[:meta][:total]

      result = OpenStruct.new
      result.devices = list
      result.meta = meta
      result
    end

    def total_devices
      path = '/devices/total'
      method = 'get'
      options = {query_params: {m2m_id: self.id}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        result
      else
        0
      end
    end
  end
end
