module Innologix
  class SlaGroup
    attr_accessor :id
    attr_accessor :name
    attr_accessor :supervisor_id
    attr_accessor :time_frames
    attr_accessor :created_at
    attr_accessor :updated_at

    attr_accessor :client
    attr_accessor :error

    def initialize(h = {})
      h.each { |k, v| public_send("#{k}=", v) }
      @client = Client.default
    end

    def list(offset = 0, limit = 10)
      path = '/sla_groups'
      method = 'get'
      options = { query_params: { offset: offset, limit: limit } }
      result = client.call_api(path, method, options)
      if result[:error].nil?
        list = []
        result[:sla_groups].each do |sla_group|
          list.push(from_hash(sla_group))
        end
        meta = OpenStruct.new
        meta.offset = result[:meta][:offset]
        meta.limit = result[:meta][:limit]
        meta.total = result[:meta][:total]

        result = OpenStruct.new
        result.sla_groups = list
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def get(id)
      path = '/sla_groups/' + id.to_s
      method = 'get'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def create
      path = '/sla_groups'
      method = 'post'
      options = { form_params: { sla_group: { name: name, supervisor_id: supervisor_id, time_frames: time_frames } } }
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update
      path = '/sla_groups/' + id.to_s
      method = 'put'
      options = { form_params: { sla_group: { name: name, supervisor_id: supervisor_id, time_frames: time_frames } } }
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def delete
      path = '/sla_groups/' + id.to_s
      method = 'delete'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def from_hash(attributes)
      sla_group = Innologix::SlaGroup.new
      sla_group.id = attributes[:id]
      sla_group.name = attributes[:name]
      sla_group.supervisor_id = attributes[:supervisor_id]
      sla_group.time_frames = attributes[:time_frames]
      sla_group.created_at = attributes[:created_at]
      sla_group.updated_at = attributes[:updated_at]
      sla_group
    end
  end
end
