module Innologix
  class Storage
    attr_accessor :id
    attr_accessor :name
    attr_accessor :ip
    attr_accessor :port
    attr_accessor :created_at
    attr_accessor :updated_at

    attr_accessor :client
    attr_accessor :error

    def initialize(h = {})
      h.each { |k, v| public_send("#{k}=", v) }
      @client = Client.default
    end

    def list(offset = 0, limit = 10)
      path = '/storages'
      method = 'get'
      options = {query_params: {offset: offset, limit: limit}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        list =[]
        result[:storages].each do |storage|
          list.push(from_hash(storage))
        end
        meta = OpenStruct.new
        meta.offset = result[:meta][:offset]
        meta.limit = result[:meta][:limit]
        meta.total = result[:meta][:total]

        result = OpenStruct.new
        result.storages = list
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def get(id)
      path = '/storages/' + id.to_s
      method = 'get'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def create
      path = '/storages'
      method = 'post'
      form_params = {name: name, ip: ip, port: port}
      options = {form_params: {storage: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update
      path = '/storages/' + id.to_s
      method = 'put'
      form_params = {name: name, ip: ip, port: port}
      options = {form_params: {storage: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def delete
      path = '/storages/' + id.to_s
      method = 'delete'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def from_hash(attributes)
      storage = Innologix::Storage.new
      storage.id = attributes[:id]
      storage.name = attributes[:name]
      storage.ip = attributes[:ip]
      storage.port = attributes[:port]
      storage.created_at = attributes[:created_at]
      storage.updated_at = attributes[:updated_at]
      storage
    end
  end
end
