module Innologix
  class Supervisor
    attr_accessor :id
    attr_accessor :name
    attr_accessor :ip
    attr_accessor :status
    attr_accessor :storage_id
    attr_accessor :created_at
    attr_accessor :updated_at

    attr_accessor :storage
    attr_accessor :groups
    attr_accessor :device_types

    attr_accessor :client
    attr_accessor :error

    def initialize(h = {})
      h.each { |k, v| public_send("#{k}=", v) }
      @client = Client.default
    end

    def list(offset = 0, limit = 10)
      path = '/supervisors'
      method = 'get'
      options = {query_params: {offset: offset, limit: limit}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        list =[]
        result[:supervisors].each do |device|
          list.push(from_hash(device))
        end
        meta = OpenStruct.new
        meta.offset = result[:meta][:offset]
        meta.limit = result[:meta][:limit]
        meta.total = result[:meta][:total]

        result = OpenStruct.new
        result.supervisors = list
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def total
      path = '/supervisors/total'
      method = 'get'
      options = {}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        result
      else
        nil
      end
    end

    def get(id)
      path = '/supervisors/' + id.to_s
      method = 'get'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def create
      path = '/supervisors'
      method = 'post'
      form_params = {name: name, ip: ip, storage_id: storage_id}
      options = {form_params: {supervisor: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update
      path = '/supervisors/' + id.to_s
      method = 'put'
      form_params = {name: name, ip: ip, storage_id: storage_id}
      options = {form_params: {supervisor: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def delete
      path = '/supervisors/' + id.to_s
      method = 'delete'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update_configs
      path = '/supervisors/' + id.to_s + '/update_configs'
      method = 'patch'
      result = client.call_api(path, method)
      if result[:error].nil?
        supervisor = from_hash(result[:supervisor])
        meta = result[:meta]
        result = OpenStruct.new
        result.supervisor = supervisor
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def restart(callback = nil)
      path = '/supervisors/' + id.to_s + '/restart'
      method = 'patch'
      form_params = {callback: callback}
      options = {form_params: form_params}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        result[:message]
      else
        RequestError.new(result)
      end
    end

    def check_status(callback = nil)
      path = '/supervisors/' + id.to_s + '/check_status'
      method = 'patch'
      form_params = {callback: callback}
      options = {form_params: form_params}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        result[:message]
      else
        RequestError.new(result)
      end
    end

    def from_hash(attributes)
      supervisor = Innologix::Supervisor.new
      supervisor.id = attributes[:id]
      supervisor.name = attributes[:name]
      supervisor.ip = attributes[:ip]
      supervisor.status = attributes[:status]
      supervisor.storage_id = attributes[:storage_id]
      supervisor.created_at = attributes[:created_at]
      supervisor.updated_at = attributes[:updated_at]

      if attributes[:storage] != nil
        supervisor.storage = Innologix::Storage.new(attributes[:storage])
      end

      supervisor.groups = []
      if attributes[:groups] != nil
        attributes[:groups].each do |group|
          supervisor.groups.push(Innologix::Group.new(group))
        end
      end

      supervisor.device_types = []
      if attributes[:device_types] != nil
        attributes[:device_types].each do |device_type|
          supervisor.device_types.push(Innologix::DeviceType.new(device_type))
        end
      end

      supervisor
    end
  end
end
