module Innologix
  class User
    attr_accessor :id
    attr_accessor :first_name
    attr_accessor :last_name
    attr_accessor :email
    attr_accessor :status
    attr_accessor :created_at
    attr_accessor :updated_at

    attr_accessor :client
    attr_accessor :error

    def initialize(h = {})
      h.each { |k, v| public_send("#{k}=", v) }
      @client = Client.default
    end

    def list(offset = 0, limit = 10)
      path = '/users'
      method = 'get'
      options = {query_params: {offset: offset, limit: limit}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        list =[]
        result[:users].each do |device|
          list.push(from_hash(device))
        end
        meta = OpenStruct.new
        meta.offset = result[:meta][:offset]
        meta.limit = result[:meta][:limit]
        meta.total = result[:meta][:total]

        result = OpenStruct.new
        result.users = list
        result.meta = meta
        result
      else
        RequestError.new(result)
      end
    end

    def get(id)
      path = '/users/' + id.to_s
      method = 'get'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def check_email(email)
      path = '/users/check_email'
      method = 'post'
      options = {form_params: {email: email}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def create
      path = '/users'
      method = 'post'
      form_params = {first_name: first_name, last_name: last_name, email: email}
      options = {form_params: {user: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def update
      path = '/users/' + id.to_s
      method = 'put'
      form_params = {first_name: first_name, last_name: last_name, email: email}
      options = {form_params: {user: form_params}}
      result = client.call_api(path, method, options)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def delete
      path = '/users/' + id.to_s
      method = 'delete'
      result = client.call_api(path, method)
      if result[:error].nil?
        from_hash(result)
      else
        RequestError.new(result)
      end
    end

    def from_hash(attributes)
      user = Innologix::User.new
      user.id = attributes[:id]
      user.first_name = attributes[:first_name]
      user.last_name = attributes[:last_name]
      user.email = attributes[:email]
      user.status = attributes[:status]
      user.created_at = attributes[:created_at]
      user.updated_at = attributes[:updated_at]
      user
    end
  end
end
